/*global dump*/
"use strict";

console.log("Loading mraid_internal.js");

(function () {
    // Root bridge object handler
    var
        mraidinternal = window.mraid_internal = {},
        bridge = window.mraidbridge,
        // Use a callback id to prevent multiple callbacks overlapping if
        // multiple callers want the same value.
        nextCallbackId = 0,
        callbacks = {},
        contactsData = {},
        handleNativeEvent = function (json) {
            if (!json) {
                return;
            }
            var callbackId = json.callbackId;
            console.log("mraidinternal handleNativeEvent listener - callbackId=" + callbackId);
            if (callbacks[callbackId]) {
                callbacks[callbackId](json.data);
            } else {
                console.log("mraidinternal - missing native handler event for callbackId: " + callbackId);
            }
        }
    ;

    //////////////////////////////////////////////////////////////////////////////////////////////////
    // MRAID get contacts

    // Register listeners to be triggered when native call completes.
    bridge.addEventListener('getContactsCompleted', function (json) {
        console.log("mraidbridge.getContactsCompleted called");
        var
            callbackId = json.callbackId,
            data = contactsData[callbackId] || []
        ;
        data = data.concat(json.data.contacts);
        if (json.data.hasMore) {
            contactsData[callbackId] = data;
        } else {
            delete contactsData[callbackId];
            json.data = {
                "contacts": data,
                "totalContacts": json.data.totalContacts
            };
            handleNativeEvent(json);
        }
    });

    bridge.addEventListener('getContactPhotosCompleted', function (json) {
        console.log("mraidbridge.getContactPhotosCompleted called");
        handleNativeEvent(json);
    });

    mraidinternal.getContacts = function (offset, maxContacts, callback) {
        // Save the callback.
        var callbackId = nextCallbackId++;
        callbacks[callbackId] = callback;
        bridge.executeNativeCallWithScheme(
            'mraidinternal',
            'getcontacts',
            'offset', offset,
            'maxContacts', maxContacts,
            'callbackId', callbackId
        );
    };

    mraidinternal.getContactPhotos = function (contactIds, callback) {
        // Save the callback.
        var callbackId = nextCallbackId++;
        callbacks[callbackId] = callback;
        bridge.executeNativeCallWithScheme(
            'mraidinternal',
            'getcontactphotos',
            'contactIds', JSON.stringify(contactIds),
            'callbackId', callbackId
        );
    };

    //////////////////////////////////////////////////////////////////////////////////////////////////
    // MRAID Local Cache

    // Register listener to be triggered when native call completes.
    bridge.addEventListener('cacheGetCompletedEvent', function (json) {
        console.log("mraidbridge.cacheGetCompletedEvent - json=" + dump(json));
        handleNativeEvent(json);
    });

    mraidinternal.cacheGet = function (key, callback) {
        // Save the callback.
        var callbackId = nextCallbackId++;
        callbacks[callbackId] = callback;
        console.log("mraidinternal.cacheGet - key=" + key + ", callbackId=" + callbackId);
        bridge.executeNativeCallWithScheme('mraidinternal', 'cacheGet', 'key', key, 'callbackId', callbackId);
    };

    mraidinternal.cacheSet = function (key, value) {
        console.log("mraidinternal.cacheSet - key=" + key + ", value=" + value);
        bridge.executeNativeCallWithScheme('mraidinternal', 'cacheSet', 'key', key, 'value', value);
    };

    mraidinternal.cacheRemove = function (key) {
        console.log("mraidinternal.cacheRemove - key=" + key);
        bridge.executeNativeCallWithScheme('mraidinternal', 'cacheRemove', 'key', key);
    };

    //////////////////////////////////////////////////////////////////////////////////////////////////
    // MRAID Facebook SDK functions

    // Register listener to be triggered when native call completes.
    bridge.addEventListener('showFacebookRequestDialogCompleted', function (json) {
        console.log("mraidbridge.showFacebookRequestDialogCompleted - json=" + JSON.stringify(json));
        handleNativeEvent(json);
    });

    mraidinternal.showFacebookRequestDialog = function (params, callback) {
        // Save the callback.
        var callbackId = nextCallbackId++;
        callbacks[callbackId] = callback;
        bridge.executeNativeCallWithScheme(
            'mraidinternal', 'showfacebookrequestdialog',
            'params', JSON.stringify(params),
            'callbackId', callbackId
        );
    };

}());
